/* 
 * This is filter.c hacked to import scores from another source 
 *
 * If a fourth argument is supplied it is read as an import file.  The
 * format of the file is
 *
 * team_name_1
 * team_name_2
 * ...
 * team_name_n
 * #
 * team|prob|time|judge       (judge=Y/N)
 *
 * It is assumed that the team name does not have '|' in it, and that the
 * judgements are given in chronological order (time is non-decreasing).
 *
 */

#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>

/* strptime() needs the following */
#define __USE_XOPEN
#include <time.h>

#define MAX_TEAMS 1000
#define MAX_TEAM_NAME 100
#define MAX_PROB 10

char team[MAX_TEAMS][MAX_TEAM_NAME], t[MAX_TEAMS];
int att[MAX_TEAMS][MAX_PROB], ptime[MAX_TEAMS][MAX_PROB], right[MAX_TEAMS];
int ttime[MAX_TEAMS], fright[MAX_TEAMS], ftime[MAX_TEAMS],  frank[MAX_TEAMS];
int i,j,k;
int nq,tn;
int rnk[MAX_TEAMS];
char buf[MAX_TEAM_NAME];
int fn,fp;
int freeze_time, length, elapsed;
time_t curr, start;

int compscore(const void *x, const void *y)
{
  int *a = (int *)x;
  int *b = (int *)y;

  if (right[*a] > right [*b]) return -1;
  if (right[*a] < right [*b]) return 1;
  if (ttime[*a] < ttime[*b]) return -1;
  if (ttime[*a] > ttime[*b]) return 1;
  return strcmp(team[*a],team[*b]);
}

/* strip trailing blanks */
void clean(char *s)
{
  int i;
  for (i=strlen(s)-1;i>=0 && s[i]==' ';i--) s[i] = 0;
}

void print(int *ord, int *rnk, char *start_time, int format){
  int i,j;

  /* print time info */
  printf("<center><table border=0>\n");
  printf("<tr><td align=right><b>Current time:</b></td><td>%s</td></tr>\n", ctime(&curr));
  printf("<tr><td align=right><b>Contest started at:</b></td>");
  if (!(*start_time)) {
    printf("<td>not started yet</td></tr>\n");
  } else {
    printf("<td>%s</td></tr>\n", start_time);
  }
  printf("<tr><td align=right><b>Contest length:</b></td>");
  printf("<td>%d:%02d:%02d</td></tr>", length/3600, (length/60)%60, length%60);
  printf("<tr><td align=right><b>Elapsed time:</b></td>");
  printf("<td>%d:%02d:%02d", elapsed/3600, (elapsed/60)%60, elapsed%60);
  if (elapsed >= freeze_time) {
    printf(" (scoreboard frozen)");
  }
  printf("</td></tr>\n");
  printf("</table></center>");
  
  
  if (format == 1) {
    printf("<center><table border=1>\n");
    
    /* generate headers */
    printf("<tr><th></th><th></th><th colspan=%d>Problems</th><th></th><th></th></tr>\n", nq);
    printf("<tr><th>Rank</th><th>Team</th>");
    for (i = 0; i < nq; i++) {
      printf("<th>%c</th>", 'A'+i);
    }
    printf("<th>Solved</th><th>Penalty</th></tr>\n");
    
    for (i = 0; i < tn; i++) {
      printf("<tr>");
      
      /* rank */
      for (j = i-1; 
	   j >= 0 && right[rnk[j]] == right[rnk[i]] && ttime[rnk[j]] == ttime[rnk[i]]; 
	   j--)
	;
      printf("<td align=center>%d</td>", j+2);
      
      /* name */
      printf("<td align=left>%s</td>", team[rnk[i]]);
      
      /* problems solved */
      for (j = 0; j < nq; j++) {
	printf("<td align=center>");
	if (att[rnk[i]][j] > 0) {
	  printf("%d/", att[rnk[i]][j]);
	  if (ptime[rnk[i]][j] >= 0) {
	    printf("%d", ptime[rnk[i]][j]);
	  } else {
	    printf("--");
	  }
	} else {
	  printf("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
	}
	
	printf("</td>");
      }
      
      /* total stats */
      printf("<td align=center>%d</td>", right[rnk[i]]);
      printf("<td align=center>%d</td>", ttime[rnk[i]]);
      
      printf("</tr>\n");
    }
    
    printf("</table></center>\n");
  } else if (format == 2) {
    char line[1024], tmp[1024];

    printf("<pre>\n");

    /* headers */
    sprintf(line, "Rank %-20s", "Team");
    for (i = 0; i < nq; i++) {
      sprintf(tmp, "   %c ", 'A'+i);
      strcat(line, tmp);
    }
    strcat(line, "  Total");
    printf("%s\n", line);
    for (i = 0; i < strlen(line)+1; i++) {
      putchar('-');
    }
    printf("\n");

    for (i = 0; i < tn; i++) {
      /* rank */
      for (j = i-1; 
	   j >= 0 && right[rnk[j]] == right[rnk[i]] && ttime[rnk[j]] == ttime[rnk[i]]; 
	   j--)
	;
      printf("%3d  ", j+2);

      /* team name */
      snprintf(line, 21, team[rnk[i]]);
      printf("%-20s", line);

      /* problems solved */
      for (j = 0; j < nq; j++) {
	if (att[rnk[i]][j] > 0 && ptime[rnk[i]][j] > 0) {
	  printf("  %3d", ptime[rnk[i]][j]);
	} else {
	  printf("     ");
	}
      }

      /* total stats */
      printf("  %d %4d\n", right[rnk[i]], ttime[rnk[i]]);
    }

    printf("</pre>\n");
  } else {
    fprintf(stderr, "Invalid format\n");
    exit(1);
  }
  
  printf("</body></html>\n");
}

int main(int argc, char *argv[])
{
  FILE *full, *data, *import;
  char buffer[10240];
  int a, b, c;
  char ch;
  struct tm t2;

  if (argc != 5 && argc != 6) {
    fprintf(stderr, "Usage: %s full.html data_file start_time <1|2> [import_file]\n", 
	    argv[0]);
    return -1;
  }

  /* get the contest length and freeze time from data file */
  if (!(data = fopen(argv[2], "r"))) {
    fprintf(stderr, "Cannot read contest data\n");
    return -1;
  }
  fgets(buffer, 10240, data);
  sscanf(buffer, "%d:%d:%d", &a, &b, &c);
  length = a*3600+b*60+c;
  fgets(buffer, 10240, data);
  sscanf(buffer, "%d:%d:%d", &a, &b, &c);
  freeze_time = a*3600+b*60+c;
  fclose(data);

  /* compute elapsed time */
  time(&curr);
  t2 = *localtime(&curr);
  strftime(buffer, 10240, "%c", &t2);
  strptime(buffer, "%c", &t2);
  curr = mktime(&t2);
  clean(argv[3]);
  if (argv[3][0]) {
    char *p;	  
    if (p = strptime(argv[3], "%n%c", &t2)) {
      start = mktime(&t2);
    } else {
      start = curr;
    }
  } else {
    start = curr;
  }
  elapsed = (int)difftime(curr, start);

  /* find out how many problems there are */
  scanf("%s",buf);
  nq = 8;
  if (isdigit(buf[strlen(buf)-1])) nq = buf[strlen(buf)-1] - '0';
  if (buf[strlen(buf)-1]>='A' && buf[strlen(buf)-1]<='J')
    nq = buf[strlen(buf)-1] -'A' + 1;
  
  /* read in team info */
  for (tn=0;1 == scanf(" \"%[^\"\n]\"",team[tn]);tn++){
    clean(team[tn]);
    right[tn] = ttime[tn] = 0;
    for (i=0;2==scanf("%d/%d",&att[tn][i],&ptime[tn][i]);i++){
      if (ptime[tn][i] >= 0) ttime[tn] += 20*att[tn][i]-20 + ptime[tn][i];
      if (ptime[tn][i] >= 0) right[tn]++;
    }
  }
  
  /* read in the full.html info, and match them with what we had so far */
  if (!(full = fopen(argv[1], "r"))) {
    fprintf(stderr, "Cannot read full.html\n");
    return -1;
  }
  while (full && 4 == fscanf(full,"%d \"%[^\"\n]\"%d%d",&j,buf,&fn,&fp)){     
    clean(buf);
    for (i=0;i<tn && strcmp(buf,team[i]);i++);
    if (i < tn) {
      fright[i] = fn;
      ftime[i] = fp;
      frank[i] = j;
    }
  }
  fclose(full);

  /* it may happen that some attempts are not counted (e.g. extra submissions
     after correct submission */
  for (i=0;i<tn;i++) {
    if (fright[i] == right[i] && ftime[i] < ttime[i]) ttime[i] = ftime[i];
  }

  /* get the import file if there is one */
  if (argc == 6) {
    if (!(import = fopen(argv[5], "r"))) {
      /* feature: fail silently if import doesn't work... */
      fprintf(stderr, "Cannot read import file\n");
    } else {
      /* read in the team names */
      while (fgets(buf, MAX_TEAM_NAME, import) && buf[0] != '#') {
	if (buf[strlen(buf)-1] != '\n') {
	  fprintf(stderr, "Team name too long: \"%s\"\n", buf);
	} else {
	  buf[strlen(buf)-1] = 0;
	}
	clean(buf);
	strcpy(team[tn], buf);
	for (i = 0; i < nq; i++) {
	  att[tn][i] = 0;
	  ptime[tn][i] = -1;
	}
	right[tn] = ttime[tn] = 0;
	tn++;
      }

      /* now read in the rest */
      while (fscanf(import, "%[^|]|%d|%d|%c\n", buf, &a, &b, &ch) == 4) {
        a--;
	if (b*60 > elapsed) {
	  continue;
	}
	clean(buf);
	for (i = 0; i < tn && strcmp(buf, team[i]); i++)
	  ;
	if (i < tn) {
	  att[i][a]++;
	  if ((ch == 'Y' || ch == 'y') && ptime[i][a] < 0) {
	    /* must have b >= ptime[i][a] because things are in order, */
            /* so we can ignore it.  We can also compute ttime now and */
	    /* not worry about changing it later on.                   */
	    ptime[i][a] = b;
	    right[i]++;
	    ttime[i] += ptime[i][a] + 20*att[i][a] - 20;
	  }
	}
      }

      fclose(import);
    }
  }

  for (i=0;i<tn;i++) rnk[i]=i;
  qsort(rnk,tn,sizeof(int),compscore);
  print(rnk,rnk,argv[3],atoi(argv[4]));

  return 0;
}
